/*
 * lber-proto.h
 * function prototypes for lber library
 */
/*
 * in abandon.c:
 */
int ldap_abandon( LDAP *ld, int msgid );

/*
 * in add.c:
 */
int ldap_add( LDAP *ld, char *dn, LDAPMod **attrs );

/*
 * in bind.c:
 */
int ldap_bind( LDAP *ld, char *who, char *passwd, int authmethod );
int ldap_unbind( LDAP *ld );

/*
 * in compare.c:
 */
int ldap_compare( LDAP *ld, char *dn, char *attr, char *value );

/*
 * in delete.c:
 */
int ldap_delete( LDAP *ld, char *dn );

/*
 * in error.c:
 */
int ldap_result2error( LDAP *ld, LDAPMessage *r, int freeit );
char *ldap_err2string( int err );
void ldap_perror( LDAP *ld, char *s );

/*
 * in modify.c:
 */
int ldap_modify( LDAP *ld, char *dn, LDAPMod **mods );

/*
 * in modrdn.c:
 */
int ldap_modrdn( LDAP *ld, char *dn, char *newrdn );

/*
 * in open.c:
 */
LDAP *ldap_open( char *host, int port );

/*
 * in parse.c:
 */
LDAPMessage *ldap_first_entry( LDAP *ld, LDAPMessage *chain );
LDAPMessage *ldap_next_entry( LDAP *ld, LDAPMessage *entry );
char *ldap_get_dn( LDAP *ld, LDAPMessage *entry );
char *ldap_dn2ufn( char *dn );
char *ldap_first_attribute( LDAP *ld, LDAPMessage *entry, BerElement **ber );
char *ldap_next_attribute( LDAP *ld, LDAPMessage *entry, BerElement *ber );
char **ldap_get_values( LDAP *ld, LDAPMessage *entry, char *target );
struct berval **ldap_get_values_len( LDAP *ld, LDAPMessage *entry, char *target );
int ldap_count_entries( LDAP *ld, LDAPMessage *chain );
int ldap_value_free( char **vals );
char **ldap_explode_dn( char *dn, int notypes );

/*
 * in result.c:
 */
int ldap_result( LDAP *ld, int msgid, int all, struct timeval *timeout,
			LDAPMessage **result );
int wait4msg( LDAP *ld, int msgid, int all, struct timeval *timeout,
			LDAPMessage **result );
int ldap_msgfree( LDAPMessage *lm );
int ldap_msgdelete( LDAP *ld, int msgid );

/*
 * in search.c:
 */
int ldap_search( LDAP *ld, char *base, int scope, char *filter, char **attrs,
			int attrsonly );

/*
 * in synchronous.c
 */
int ldap_add_s( LDAP *ld, char *dn, LDAPMod **attrs );
int ldap_delete_s( LDAP *ld, char *dn );
int ldap_modify_s( LDAP *ld, char *dn, LDAPMod **mods );
int ldap_modrdn_s( LDAP *ld, char *dn, char *newrdn );
int ldap_search_s( LDAP *ld, char *base, int scope, char *filter,
	char **attrs, int attrsonly, LDAPMessage **res );
int ldap_compare_s( LDAP *ld, char *dn, char *attr, char *value );
