/*
 *  Copyright (c) 1990 Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  open.c
 */

#ifndef lint 
static char copyright[] = "@(#) Copyright (c) 1990 Regents of the University of Michigan.\nAll rights reserved.\n";
#endif

#include "lber.h"
#include "ldap.h"
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#ifndef INADDR_LOOPBACK
#define INADDR_LOOPBACK	((u_long) 0x7f00000001)
#endif

#ifdef LDAP_DEBUG
int	ldap_debug;
#endif

/*
 * ldap_open - initialize and connect to an ldap server.  A magic cookie to
 * be used for future communication is returned on success, NULL on failure.
 *
 * Example:
 *	LDAP	*ld;
 *	ld = ldap_open( hostname, port );
 */

LDAP *ldap_open( host, port )
char	*host;
int	port;
{
	int 			s;
	unsigned long		address;
	struct sockaddr_in 	sock;
	struct hostent		*hp;
	LDAP			*ld;
	char			*p, hostname[BUFSIZ];

	Debug( LDAP_DEBUG_TRACE, "ldap_open\n", 0, 0, 0 );

	if ( host != NULL ) {
		if ( (address = inet_addr( host )) == -1 ) {
			if ( (hp = gethostbyname( host )) == NULL ) {
				perror( "socket" );
				return( NULL );
			}
			(void) memcpy( (char *) &address, (char *) hp->h_addr,
			    sizeof(unsigned long) );
			address = ntohl( address );
			strcpy( hostname, hp->h_name );
		}
	} else {
		address = INADDR_LOOPBACK;
		(void) gethostname( hostname, sizeof(hostname) );
	}
	if ( (p = strchr( hostname, '.' )) != NULL )
		*p = '\0';
	if ( port == 0 )
		port = LDAP_PORT;

	if ( (s = socket( AF_INET, SOCK_STREAM, 0 )) < 0 ) {
		perror( "socket" );
		return( NULL );
	}

	sock.sin_addr.s_addr = htonl( address );
	sock.sin_family = AF_INET;
	sock.sin_port = htons( port );

	if ( connect( s, (struct sockaddr *) &sock, sizeof(sock) ) < 0 ) {
		perror( "connect" );
		return( NULL );
	}

	if ( (ld = (LDAP *) calloc( sizeof(LDAP), 1 )) == NULL ) {
		close( s );
		return( NULL );
	}
	ld->ld_sb.sb_sd = s;
	ld->ld_host = strdup( hostname );

	return( ld );
}
