/*
 *  Copyright (c) 1990 Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  open.c
 */

#ifndef lint 
static char copyright[] = "@(#) Copyright (c) 1990 Regents of the University of Michigan.\nAll rights reserved.\n";
#endif

#include "lber.h"
#include "ldap.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "macos.h"

#ifdef LDAP_DEBUG
int	ldap_debug;
#endif

/*
 * ldap_open - initialize and connect to an ldap server.  A magic cookie to
 * be used for future communication is returned on success, NULL on failure.
 *
 * Example:
 *	LDAP	*ld;
 *	ld = ldap_open( hostname, port );
 */

LDAP *ldap_open( host, port )
char	*host;
int	port;
{
	short 			s, i;
	unsigned long	address;
    struct hostInfo	*hip;
	LDAP			*ld;

	Debug( LDAP_DEBUG_TRACE, "ldap_open\n", 0, 0, 0 );

    if (( hip = gethostinfobyname( host )) == NULL ) {
		return( NULL );
    }
    
 	if ( port == 0 )
		port = LDAP_PORT;

	if (( s = tcpopen( NULL, TCP_BUFSIZ )) < 0 ) {
		Debug( LDAP_DEBUG_TRACE, "tcpopen failed\n", 0, 0, 0 );
		return( NULL );
	}

    for ( i = 0; hip->addr[ i ] != 0; ++i ) {
		(void) memcpy( (char *)&address, (char *)&hip->addr[ i ], sizeof( long ));
		if ( tcpconnect( s, address, port ) > 0 ) {
			break;
		}
	}
	
	if (  hip->addr[ i ] == 0 ) {
		tcpclose( s );
		Debug( LDAP_DEBUG_TRACE, "tcpconect failed\n", 0, 0, 0 );
		return( NULL );
	}

	if ( (ld = (LDAP *) calloc( sizeof(LDAP), 1 )) == NULL ) {
		tcpclose( s );
		return( NULL );
	}
	ld->ld_sb.sb_sd = s;

	return( ld );
}
