/*
 * Copyright (c) 1991,1992 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 *
 * The University of Michigan would like to thank the following people for
 * their contributions to this piece of software:
 *
 *	Robert Urquhart    <robert@sfu.ca>
 *	Simon Fraser University, Academic Computing Services
 */

#include <stdio.h>
#include <string.h>
#ifndef DOS
#include <sgtty.h>
#endif
#include <ctype.h>
#include <signal.h>
#include <setjmp.h>
#if defined(NeXT)
#include <stdlib.h>
#endif
#include <memory.h>
#include <lber.h>
#include <ldap.h>
#include "ud.h"

#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1991,1992 Regents of the University of Michigan.\nAll rights reserved.\n";
#endif

static char server[BUFSIZ]; 	/* LDAP server name */

char default_base[BUFSIZ];	/* default search base set at start up */
extern char base[];		/* search base */
extern char me[];		/* string containing result of I AM */
static jmp_buf env;		/* spot to jump to on an interrupt */
int lpp;			/* lines per page */
int debug;			/* debug flag */
int verbose;			/* 1 if verbose mode on */
LDAP *ld;			/* LDAP descriptor */

main(argc, argv)
int argc;
char *argv[];
{
	extern char Version[];			/* version number */
	extern char *optarg;			/* for parsing argv */
	register int c;				/* for parsing argv */
	register char *cp;			/* for parsing Version */
	extern void exit();

	/*  handle argument list */
	while ((c = getopt(argc, argv, "d:s:v")) != -1)
		switch (c) {
		case 'd' :
#ifdef LDAP_DEBUG
			ldap_debug = debug = atoi(optarg);
#else
			debug = atoi(optarg);
#endif
			break;
		case 's' :
			strcpy(server, optarg);
			break;
		case 'v' :
			verbose = 1;
			break;
		default:
			fprintf(stderr, "Usage: %s [-d debug-level] [-s server] [-v]\n",
					 argv[0]);
			exit(-1);
			/* NOTREACHED */
		}
	initialize_client();

	/* just print the first line of Version[] */
	cp = strchr(Version, '\t');
	if (cp != NULL)
		*cp = '\0';
	printf(Version);

	/* now tackle the user's commands */
	do_commands();
	/* NOTREACHED */
}

do_commands()
{
	LDAPMessage *mp;			/* returned by find() */
	register char *cp;			/* misc char pointer */
	register char *ap;			/* misc char pointer */
	static char cmd[BUFSIZ];		/* holds the command */
	static char input[BUFSIZ];		/* buffer for fgets() */
	extern struct entry Person;		/* holds info about someone */
	extern char *alias();
	extern LDAPMessage *find();

	if (verbose) 
		printf("\n  Enter a command.  If you need help, type 'h' or '?' and hit RETURN.\n");
	/* jump here on an interrupt */
	(void) setjmp(env);
	for (;;) {
		printf("* ");
		fflush(stdout);
		cp = input;
		fetch_buffer(input, sizeof(input), stdin);
		if (*input == '\0') {
			putchar('\n');
			continue;
		}
		while (isspace(*cp))
			cp++;	
		ap = cmd;
		if (memset(cmd, '\0', sizeof(cmd)) == NULL)
			fatal("memset");
		while (!isspace(*cp) && (*cp != '\0'))
			*ap++ = *cp++;
		if (!strncasecmp("status", cmd, strlen(cmd)))
			status();
		else if (!strncasecmp("stop", cmd, strlen(cmd)) ||
			!strncasecmp("quit", cmd, strlen(cmd)))
			break;
		else if (!strncasecmp("cb", cmd, 2) ||
			 !strncasecmp("cd", cmd, 2) ||
			 !strncasecmp("moveto", cmd, strlen(cmd))) {
			while (isspace(*cp) && (*cp != '\0')) cp++;
			if (!strncasecmp(cp, "base", 4))
				cp += 4;
			while (isspace(*cp) && (*cp != '\0')) cp++;
			if (*cp == '\0') {
				printf("  Syntax error\n");
				continue;
			}
			change_base(cp);
		}
		else if (!strncasecmp("find", cmd, strlen(cmd)) ||
			 !strncasecmp("display", cmd, strlen(cmd)) ||
			 !strncasecmp("show", cmd, strlen(cmd))) {
			while (isspace(*cp))
				cp++;
			if (*cp == '\0') {
				printf("  Syntax error\n");
				continue;
			}
			if ((cp = alias(cp)) == NULL)
				continue;
			if ((mp = find(cp)) != NULL) {
				parse_answer(mp);
				print_person();
				ldap_msgfree(mp);
			}
		}
		else if (!strncasecmp("modify", cmd, strlen(cmd)) ||
			 !strncasecmp("change", cmd, strlen(cmd)) ||
			 !strncasecmp("alter", cmd, strlen(cmd))) {
			while (isspace(*cp))
				cp++;
			(void) modify(alias(cp));
		}
		else if (!strncasecmp("bind", cmd, strlen(cmd))) {
			while (isspace(*cp))
				cp++;
			(void) auth(alias(cp), (char *) NULL);
		}
		else if ((cmd[0] == '?') || !strncasecmp("help", cmd, strlen(cmd))) {
			while (isspace(*cp))
				cp++;
			print_help(cp);
		}
		else if (!strncasecmp("iam", cmd, strlen(cmd))) {
			while (isspace(*cp))
				cp++;
			iam(cp);
		}
		else if (!strncasecmp("whoami", cmd, strlen(cmd))) {
			print_my_name();
		}
		else if (!strncasecmp("verbose", cmd, strlen(cmd))) {
			verbose = 1 - verbose;
			/*  ;-)  */
			if (verbose)
				printf("  Verbose mode has been turned on.\n");
		}
		else if (cmd[0] == '\0') {
			putchar('\n');
		}
		else {
			printf("  Invalid command.  Type \"help commands.\"\n");
		}
	}
	printf(" Thank you!\n");
	
	ldap_unbind(ld);
	exit(0);
	/* NOTREACHED */
}

status()
{
	void printbase();

	printf("  Current server is %s\n", server);
	printbase("  Search base is ", base);
	bind_info();
}

change_base(s)
char *s;
{
	register char *cp, *np;			/* utility pointers */
	char **rdns;				/* for parsing */
	int num_picked;				/* # of selected base */
	int j;					/* used with num_picked */
	int i = 1;				/* index into choices array */
	int matches;				/* # of matches found */
	int rest = 1;				/* # left to display */
	static char *choices[BUFSIZ];		/* bases from which to choose */
	static char resp[16];			/* for prompting user */
	void printbase();
	static char *attrs[] = { OBJECT_CLASS, NULL };
	extern char * friendly_country();
	LDAPMessage *mp;			/* results from a search */
	LDAPMessage *ep;			/* for going thru bases */
	extern void exit();

	/* sanity check -- should never happen */
	if (s == NULL) {
		fprintf(stderr, "NULL base!\n");
		exit(0);
	}

	/* user forgot to supply an argument on the change base command */
	if ((*s == '\n') || (*s == '\0'))
		return;

	if (!strcasecmp(s, "root")) {
		base[0] = '\0';
		printbase("  Search base is ", base);
		return;
	}

	/*
	 *  User wants to ascend one level in the X.500 tree.
	 *  Easy:  Just strip off the first element of the
	 *  current search base, unless it's the root, in
	 *  which case we just do nothing.
	 */
	if (!strcasecmp(s, "..")) {
		if (base[0] != '\0') {
			cp = np = base;
			cp = strchr(cp, '=');
			cp++;
			if ((cp = strchr(cp, '=')) == NULL)	/* one elem */
				base[0] = '\0';
			else {
				while(!isspace(*cp))
					cp--;
				cp++;
				while (*cp != '\0')
					*np++ = *cp++;
				*np = '\0';
			}
			printbase("  Search base is now ", base);
		}
		else
			printf("  You are already at the root\n");
		return;
	}

	/* user wants to see what is directly below this level */
	if (*s == '?') {
		/*
		 *  Fetch the list of entries directly below this level.
		 *  Once we have the list, we will print it for the user, one
		 *  screenful at a time.  At the end of each screen, we ask
		 *  the user if they want to see more.  They can also just
		 *  type a number at that point too.
		 */
		if (ldap_search_s(ld, base, LDAP_SCOPE_ONELEVEL, "(|(objectClass=quipuNonLeafObject)(objectClass=externalNonLeafObject))", attrs, FALSE, &mp) != LDAP_SUCCESS) {
			ldap_perror(ld, "ldap_search_s");
			return;
		}
		if ((matches = ldap_count_entries(ld, mp)) < 1) {
			printf("  There is nothing below this level.\n");
			(void) ldap_msgfree(mp);
			return;
		}
		num_picked = 0;
		printf(" There are %d choices:\n", matches);
		for (ep = ldap_first_entry(ld, mp); ep != NULL; ep = ldap_next_entry(ld, ep)) {
			/*
			 *  Put the last component of the DN into 'lastDN'.
			 *  If we are at the root level, convert any country
			 *  codes to recognizable names for printing.
			 */
			choices[i] = ldap_get_dn(ld, ep);
			rdns = ldap_explode_dn(choices[i], TRUE);
			printf(" %2d. %s\n", i, *rdns);
			(void) ldap_value_free(rdns);
			i++;
			if ((rest++ > (lpp - 3)) && (i < matches)) {
				printf("More? ");
				fflush(stdout);
				fetch_buffer(resp, sizeof(resp), stdin);
				if ((resp[0] == 'n') || (resp[0] == 'N'))
					break;
				else if (((num_picked = atoi(resp)) != 0) && (num_picked < i))
					break;
				rest = 1;
			}
		}
		for (;;) {
			if (num_picked != 0) {
				j = num_picked;
				num_picked = 0;
			}
			else {
				printf(" Which number? ");
				fflush(stdout);
				fetch_buffer(resp, sizeof(resp), stdin);
				j = atoi(resp);
			}
			if (j == 0) {
				(void) ldap_msgfree(mp);
				for (i = 0; i < matches; i++)
					(void) free(choices[i]);
				return;
			}
			if ((j < 1) || (j >= i))
				printf(" Invalid number\n");
			else {
				strcpy(base, choices[j]);
				printbase("  Search base is ", base);
				(void) ldap_msgfree(mp);
				for (i = 0; i < matches; i++)
					(void) free(choices[i]);
				return;
			}
		}
	}
	/* set the search base back to the original default value */
	else if (!strcasecmp(s, "default")) {
		strcpy(base, default_base);
		printbase("  Search base is ", base);
	}
	/* they typed in something -- see if it is legit */
	else {
		/* user cannot do something like 'cb 33' */
		if (atoi(s) != 0) {
			printf("  \"%s\" is not a valid search base\n", s);
			printf("  Search base unchanged.\n");
			printf("  Try using 'cb ?'\n");
			return;
		}
		if (vrfy_the_base(s)) {
			strcpy(base, s);
			printbase("  Search base is ", base);
		}
		else {
			printf("  \"%s\" is not a valid search base\n", s);
			printf("  Search base unchanged.\n");
		}
	}
}

initialize_client()
{
	FILE *fp;				/* for config file */
	static char buffer[BUFSIZ];		/* for fgets() */
	register char *cp;			/* for fiddling with fgets() */
	char *term;				/* for tty set-up */
	static char bp[1024];			/* for tty set-up */
	extern char bound_as[];			/* who we are bound as */
	extern SIG_FN attn();			/* ^C signal handler */
	extern void exit();
	extern char *getenv();

	/*
	 *  If there is a config file, read it.
	 *
	 *  Could have lines that look like this:
	 *
	 *	server <ip-address or domain-name>
	 *	base   <default search base>
	 */
	strcpy(default_base, DEFAULT_BASE);
	if ((fp = fopen(CONFIG_FILE, "r")) != NULL) {
		while (fgets(buffer, sizeof(buffer), fp) != NULL) {
			if (!strncmp(buffer, "server", 6)) {
				if (server[0] == '\0') {
					cp = buffer + 6;
					while (isspace(*cp))
						cp++;
					if ((*cp == '\0') || (*cp == '\n'))
						continue;
					(void) strncpy(server, cp, sizeof(server) - 1);
					if ((cp = strchr(server, '\n')) != NULL)
						*cp = '\0';
				}
			}
			else if (!strncmp(buffer, "base", 4)) {
				cp = buffer + 4;
				while (isspace(*cp))
					cp++;
				if ((*cp == '\0') || (*cp == '\n'))
					continue;
				(void) strncpy(default_base, cp, sizeof(default_base) - 1);
				if ((cp = strchr(default_base, '\n')) != NULL)
					*cp = '\0';
			}
			else
				fprintf(stderr, "?? -> %s\n", buffer);
		}
	}
	if (server[0] == '\0')
		strcpy(server, DEFAULT_SERVER);

	/*
	 *  Set up our DIXIE connection.  The values of retry and timeout
	 *  are meaningless since we will immediately be doing a null bind
	 *  because we want to be sure to use TCP, not UDP.
	 */
	if ((ld = ldap_open(server, LDAP_PORT)) == NULL) {
		ldap_perror(ld, "ldap_open");
		exit(0);
		/* NOTREACHED */
	}
	if (ldap_bind(ld, (char *) UD_USER, (char *) UD_PASSWD, UD_AUTH_METHOD) < 0) {
		ldap_perror(ld, "ldap_bind");
		exit(0);
		/* NOTREACHED */
	}
	(void) strcpy(bound_as, "Nobody");

	/* set the initial search base */
	strcpy(base, default_base);

	/* terminal initialization stuff goes here */
	lpp = DEFAULT_LINES;

	(void) signal(SIGINT, attn);

#if !defined(DOS) && !defined(NOTERMCAP)
	{
	struct winsize win;			/* for tty set-up */
	extern SIG_FN chwinsz();		/* WINSZ signal handler */

	if (((term = getenv("TERM")) == NULL) || (tgetent(bp, term) <= 0))
		return;
	else {
		if (ioctl(fileno(stdout), TIOCGWINSZ, &win) < 0) {
			lpp = tgetnum("li");
		}
		else {
			if ((lpp = win.ws_row) == 0)
				lpp = tgetnum("li");
			if ((lpp <= 0) || tgetflag("hc"))
				lpp = DEFAULT_LINES;
		}
	}
	(void) signal(SIGWINCH, chwinsz);

	}
#endif
}

SIG_FN attn()
{
	fflush(stderr);
	fflush(stdout);
	printf("\n\n  INTERRUPTED!\n");
#if defined(DOS) || defined(SYSV)
	(void) signal(SIGINT, attn);
#endif
	longjmp(env, 1);
}

#if !defined(DOS) && !defined(NOTERMCAP)
SIG_FN chwinsz() 
{
	struct winsize win;

	(void) signal(SIGWINCH, SIG_IGN);
	if (ioctl(fileno(stdout), TIOCGWINSZ, &win) != -1) {
		if (win.ws_row != 0)
			lpp = win.ws_row;
	}
	(void) signal(SIGWINCH, chwinsz);
}
#endif

char * alias(s)
char *s;
{
	if (!strncasecmp("me", s, 2) && (*(s + 2) == '\0')) {
		if (me[0] == '\0') {
			printf("  I don't know who you are.\n");
			return(NULL);
		}
		else
			return(me);
	}
	else if (*s == '\0')
		return(NULL);
	else
		return(s);
}
