/*
 * Copyright (c) 1991 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#ifndef __STDC__
#include <memory.h>
#endif
#include <lber.h>
#include <ldap.h>
#include "ud.h"

struct entry Person;
extern LDAP *ld;

parse_answer(s)
LDAPMessage *s;
{
	char **rdns;
	register int i;
	BerElement *cookie;
	register LDAPMessage *ep;
	register char *ap, **vp, **tp;

	/* initialize */
	(void) memset((char *) &Person, 0, sizeof(Person));

	for (ep = ldap_first_entry(ld, s); ep != NULL; ep = ldap_next_entry(ld, ep)) {
		rdns = ldap_explode_dn(ldap_get_dn(ld, ep), TRUE);
		strcpy(Person.name, *rdns);
		for (ap = ldap_first_attribute(ld, ep, &cookie); ap != NULL; ap = ldap_next_attribute(ld, ep, cookie)) {
			if (!strcasecmp(ap, TITLE)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.title[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, POSTAL_ADDRESS)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.baddr[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, PHONE_NUMBER)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.bphon[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, E_MAIL_ADDRESS)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.email[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, HOME_PHONE)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.hphon[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, HOME_ADDRESS)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.haddr[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, MLDESCRIPTION)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.mldsc[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, DESCRIPTION)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.descr[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, OBJECT_CLASS)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.class[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, OWNER)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.owner[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, MEMBERS)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.mmbrs[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, FAX)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.faxxx[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, UNIQNAME)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++)
					strcpy(Person.uid_x[i++], *tp);
				ldap_value_free(vp);
			}
			else if (!strcasecmp(ap, COMMON_NAME)) {
				i = 0;
				vp = (char **) ldap_get_values(ld, ep, ap);
				for (tp = vp; *tp != NULL; tp++) {
					if (!strcmp(*tp, Person.name))
						continue;
					strcpy(Person.names[i++], *tp);
				}
				ldap_value_free(vp);
			}
		}
	}
}

print_person()
{
	printf(" \"%s\"\n", Person.name);
	if (Person.names[0][0] != '\0')
		print_entry(Person.names, "Also known as:     ");
	if (Person.email[0][0] != '\0')
		print_entry(Person.email, "E-mail address:    ");
	if (Person.faxxx[0][0] != '\0')
		print_entry(Person.faxxx, "Fax number:        ");
	if (Person.bphon[0][0] != '\0')
		print_entry(Person.bphon, "Business phone:    ");
	if (Person.baddr[0][0] != '\0')
		print_entry(Person.baddr, "Business address:  ");
	if (Person.hphon[0][0] != '\0')
		print_entry(Person.hphon, "Home phone:        ");
	if (Person.haddr[0][0] != '\0')
		print_entry(Person.haddr, "Home address:      ");
	if (Person.title[0][0] != '\0')
		print_entry(Person.title, "Title:             ");
	if (Person.owner[0][0] != '\0')
		print_entry(Person.owner, "Owner:             ");
	if (Person.descr[0][0] != '\0')
		print_entry(Person.descr, "Short description: ");
	if (Person.mldsc[0][0] != '\0')
		print_entry(Person.mldsc, "Description:       ");
	if (Person.mmbrs[0][0] != '\0')
		print_entry(Person.mmbrs, "Members:           ");
	if (Person.uid_x[0][0] != '\0')
		print_entry(Person.uid_x, "Uniqname:          ");
}

print_entry(s, label)
char s[][MAX_VAL_SIZE], *label;
{
	register int i, j, k;
	register char c;
	int lead = strlen(label) + 2;

	printf("  %s", label);
	for (i = 0; s[i][0] != '\0'; i++) {
		if (i > 0) {
			if (!strncmp(label, "Home a", 6) || !strncmp(label, "Business a", 10)) {
				printf("  %s", label);
			}
			else {
				for (k = lead; k > 0; k--)
					putchar(' ');
			}
		}
		for (j = 0; s[i][j] != '\0'; j++) {
			c = s[i][j];
			switch (c) {
			case '$' :
				putchar('\n');
				for (k = lead; k > 0; k--)
					putchar(' ');
				j++;
				break;
			case '\n' :
				putchar('%');
				putchar('\n');
				break;
			default:
				putchar(c);
			}
		}
		putchar('\n');
	}
	return;
}
